/* Copyright (C) 1989 Aladdin Enterprises.  All rights reserved.
   Distributed by Free Software Foundation, Inc.

This file is part of Ghostscript.

Ghostscript is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the Ghostscript General Public License for full details.

Everyone is granted permission to copy, modify and redistribute
Ghostscript, but only under the conditions described in the Ghostscript
General Public License.  A copy of this license is supposed to have been
given to you along with Ghostscript so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.  */

/* gspaint.c */
/* Painting procedures for GhostScript library */
#include "gx.h"
#include "malloc_.h"
#include "gxfixed.h"
#include "gxmatrix.h"			/* for gs_state */
#include "gspaint.h"
#include "gzpath.h"
#include "gzstate.h"
#include "gzdevice.h"

/* Erase the page */
int
gs_erasepage(gs_state *pgs)
{	device *pdev = pgs->device;
	gx_device *dev = pdev->info;
	(*dev->procs->fill_rectangle)(dev, 0, 0, dev->width, dev->height, pdev->white);
	return 0;
}

/* Fill using the winding number rule */
int
gs_fill(gs_state *pgs)
{	int code;
	gx_color_render(pgs->color, pgs);
	code = gx_fill_path(pgs->path, pgs->color, pgs, gx_rule_winding_number);
	if ( !code ) gs_newpath(pgs);
	return code;
}

/* Fill using the even/odd rule */
int
gs_eofill(gs_state *pgs)
{	int code;
	gx_color_render(pgs->color, pgs);
	code = gx_fill_path(pgs->path, pgs->color, pgs, gx_rule_even_odd);
	if ( !code ) gs_newpath(pgs);
	return code;
}

/* Stroke the current path */
int
gs_stroke(gs_state *pgs)
{	int code;
	gx_color_render(pgs->color, pgs);
	code = gx_stroke_fill(pgs->path, pgs);
	if ( !code ) gs_newpath(pgs);
	return code;
}

/* Compute the stroked outline of the current path */

int
gs_strokepath(gs_state *pgs)
{	gx_path spath;
	int code;
	gx_path_init(&spath, &pgs->memory_procs);
	code = gx_stroke_add(pgs->path, &spath, pgs);
	if ( code < 0 ) return code;
	gx_path_release(pgs->path);
	*pgs->path = spath;
	return 0;
}

/* Render a sampled image */
int
gs_image(gs_state *pgs, int width, int height, int bps,
  gs_matrix *pmat, int (*proc)(P2(byte **, int *)))
{	gs_image_enum *penum = (gs_image_enum *)malloc(gs_image_enum_sizeof);
	int code;
	byte *data;
	int size;
	if ( (code = gs_image_init(penum, pgs, width, height, bps, pmat)) < 0 )
	   {	free((char *)penum);
		return code;
	   }
	while ( (code = (*proc)(&data, &size)) == 0 )
	   {	if ( (code = gs_image_next(penum, data, size)) < 0 )
			break;
		if ( code > 0 )
		   {	code = 0;
			break;
		   }
	   }
	free((char *)penum);
	return code;
}

/* Render a mask */
int
gs_imagemask(gs_state *pgs, int width, int height, int invert,
  gs_matrix *pmat, int (*proc)(P2(byte **, int *)))
{	gs_image_enum *penum = (gs_image_enum *)malloc(gs_image_enum_sizeof);
	int code;
	byte *data;
	int size;
	if ( (code = gs_imagemask_init(penum, pgs, width, height, invert, pmat)) < 0 )
	   {	free((char *)penum);
		return code;
	   }
	while ( (code = (*proc)(&data, &size)) == 0 )
	   {	if ( (code = gs_image_next(penum, data, size)) < 0 )
			break;
		if ( code > 0 )
		   {	code = 0;
			break;
		   }
	   }
	free((char *)penum);
	return code;
}
